@@ -67,6 +67,9 @@ struct connman_network {
 		char *private_key_path;
 		char *private_key_passphrase;
 		char *phase2_auth;
+		connman_bool_t wps;
+		connman_bool_t use_wps;
+		char *pin_wps;
 	} wifi;
 };
 
@@ -161,6 +164,14 @@ static void network_destruct(struct connman_element *element)
 	g_free(network->wifi.mode);
 	g_free(network->wifi.security);
 	g_free(network->wifi.passphrase);
+	g_free(network->wifi.eap);
+	g_free(network->wifi.identity);
+	g_free(network->wifi.ca_cert_path);
+	g_free(network->wifi.client_cert_path);
+	g_free(network->wifi.private_key_path);
+	g_free(network->wifi.private_key_passphrase);
+	g_free(network->wifi.phase2_auth);
+	g_free(network->wifi.pin_wps);
 
 	g_free(network->group);
 	g_free(network->node);
@@ -580,6 +591,22 @@ void connman_network_set_error(struct connman_network *network,
 	}
 }
 
+void connman_network_clear_error(struct connman_network *network)
+{
+	struct connman_service *service;
+
+	DBG("network %p", network);
+
+	if (network == NULL)
+		return;
+
+	if (network->connecting == TRUE || network->associating == TRUE)
+		return;
+
+	service = __connman_service_lookup_from_network(network);
+	__connman_service_clear_error(service);
+}
+
 static void set_configuration(struct connman_network *network)
 {
 	struct connman_service *service;
@@ -738,15 +765,8 @@ static gboolean set_connected(gpointer user_data)
 	DBG("service %p ipv4 %p ipv6 %p", service, ipconfig_ipv4,
 		ipconfig_ipv6);
 
-	if (ipconfig_ipv4)
-		ipv4_method = __connman_ipconfig_get_method(ipconfig_ipv4);
-	else
-		ipv4_method = CONNMAN_IPCONFIG_METHOD_UNKNOWN;
-
-	if (ipconfig_ipv6)
-		ipv6_method = __connman_ipconfig_get_method(ipconfig_ipv6);
-	else
-		ipv6_method = CONNMAN_IPCONFIG_METHOD_UNKNOWN;
+	ipv4_method = __connman_ipconfig_get_method(ipconfig_ipv4);
+	ipv6_method = __connman_ipconfig_get_method(ipconfig_ipv6);
 
 	DBG("method ipv4 %d ipv6 %d", ipv4_method, ipv6_method);
 	DBG("network connected %d", network->connected);
@@ -757,6 +777,7 @@ static gboolean set_connected(gpointer user_data)
 		switch (ipv6_method) {
 		case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
 		case CONNMAN_IPCONFIG_METHOD_OFF:
+		case CONNMAN_IPCONFIG_METHOD_AUTO:
 			break;
 		case CONNMAN_IPCONFIG_METHOD_FIXED:
 		case CONNMAN_IPCONFIG_METHOD_MANUAL:
@@ -774,6 +795,7 @@ static gboolean set_connected(gpointer user_data)
 		switch (ipv4_method) {
 		case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
 		case CONNMAN_IPCONFIG_METHOD_OFF:
+		case CONNMAN_IPCONFIG_METHOD_AUTO:
 			return FALSE;
 		case CONNMAN_IPCONFIG_METHOD_FIXED:
 			if (set_connected_fixed(network) < 0) {
@@ -1026,6 +1048,7 @@ int __connman_network_clear_ipconfig(struct connman_network *network,
 	case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
 	case CONNMAN_IPCONFIG_METHOD_OFF:
 	case CONNMAN_IPCONFIG_METHOD_FIXED:
+	case CONNMAN_IPCONFIG_METHOD_AUTO:
 		return -EINVAL;
 	case CONNMAN_IPCONFIG_METHOD_MANUAL:
 		connman_element_unregister_children_type(&network->element,
@@ -1056,6 +1079,7 @@ int __connman_network_set_ipconfig(struct connman_network *network,
 		switch (method) {
 		case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
 		case CONNMAN_IPCONFIG_METHOD_OFF:
+		case CONNMAN_IPCONFIG_METHOD_AUTO:
 			break;
 		case CONNMAN_IPCONFIG_METHOD_FIXED:
 		case CONNMAN_IPCONFIG_METHOD_MANUAL:
@@ -1078,6 +1102,7 @@ int __connman_network_set_ipconfig(struct connman_network *network,
 		case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
 		case CONNMAN_IPCONFIG_METHOD_OFF:
 		case CONNMAN_IPCONFIG_METHOD_FIXED:
+		case CONNMAN_IPCONFIG_METHOD_AUTO:
 			return -EINVAL;
 		case CONNMAN_IPCONFIG_METHOD_MANUAL:
 			return manual_ipv4_set(network, ipconfig_ipv4);
@@ -1229,6 +1254,9 @@ int connman_network_set_string(struct connman_network *network,
 	} else if (g_str_equal(key, "WiFi.Phase2") == TRUE) {
 		g_free(network->wifi.phase2_auth);
 		network->wifi.phase2_auth = g_strdup(value);
+	} else if (g_str_equal(key, "WiFi.PinWPS") == TRUE) {
+		g_free(network->wifi.pin_wps);
+		network->wifi.pin_wps = g_strdup(value);
 	}
 
 	err = connman_element_set_string(&network->element, key, value);
@@ -1282,6 +1310,8 @@ const char *connman_network_get_string(struct connman_network *network,
 		return network->wifi.private_key_passphrase;
 	else if (g_str_equal(key, "WiFi.Phase2") == TRUE)
 		return network->wifi.phase2_auth;
+	else if (g_str_equal(key, "WiFi.PinWPS") == TRUE)
+		return network->wifi.pin_wps;
 
 	return connman_element_get_string(&network->element, key);
 }
@@ -1301,6 +1331,10 @@ int connman_network_set_bool(struct connman_network *network,
 
 	if (g_strcmp0(key, "Roaming") == 0)
 		return connman_network_set_roaming(network, value);
+	else if (g_strcmp0(key, "WiFi.WPS") == 0)
+		network->wifi.wps = value;
+	else if (g_strcmp0(key, "WiFi.UseWPS") == 0)
+		network->wifi.use_wps = value;
 
 	return connman_element_set_bool(&network->element, key, value);
 }
@@ -1319,6 +1353,10 @@ connman_bool_t connman_network_get_bool(struct connman_network *network,
 
 	if (g_str_equal(key, "Roaming") == TRUE)
 		return network->roaming;
+	else if (g_str_equal(key, "WiFi.WPS") == TRUE)
+		return network->wifi.wps;
+	else if (g_str_equal(key, "WiFi.UseWPS") == TRUE)
+		return network->wifi.use_wps;
 
 	return connman_element_get_bool(&network->element, key);
 }
